"""Top-level package for ati18n."""

OUTPUT_HEAD = {'tools':'ati18n', 'version':'0.1.0', 'app_type':'java'}
ERROR_1001 = {'no':'1001', 'level':'错误', 'scope':'整体翻译', 'name':'翻译条目数不匹配', 'data':'检查的数据文件和相关信息', "comment":''}
ERROR_1002 = {'no':'1002', 'level':'错误', 'scope':'整体翻译', 'name':'文件名指定的语言种类和文件内容不匹配', 'data':'检查的数据文件和相关信息', "comment":''}
ERROR_2001 = {'no':'2001', 'level':'警告/错误', 'scope':'单个翻译', 'name':'翻译项和指定的语言类型不匹配', 'data':'检查的数据文件和相关信息', "comment":''}

from .ati18n import *



__author__ = 'painterg'
__email__ = '22396997@qq.com'

import py3langid as langid
import time, datetime, pytz
from .common import *

def get_now():
        utc = pytz.timezone('Asia/Shanghai')
        t = datetime.datetime.now(tz=utc).strftime('%Y-%m-%d %H:%M:%S')
        return t 


def determine_lang(file_name, value, app_type):
        lang = langid.classify(value)
        lang_type = lang[0]
        if (lang_type == 'zh' and app_type == 'Java'):
            lang_type = 'zh_CN'
        end = file_name.find('.properties')
        start = end - len(lang_type)
        type_from_file_name=file_name[start:end]
        if (lang_type == type_from_file_name):
            return (True, lang_type)
        else:
            return (False, lang_type)


def out_template_file(file_name, error):
    data = OutputDataSimple(type=DataType.File.value, file_path=file_name)
    result = OutputResult(No=error['no'], Level=error['level'], Scope=error['scope'], Name=error['name'], Data=data.json, Comment=error['comment'])
    return result


def out_template_item(file_name, error, key, value, code):
    data = OutputDataSimple(type=DataType.File.value, file_path=file_name)
    data = OutputDataItem(type=DataType.Item.value, file_path=file_name, key_name=key, key_value=value, lang_code=code, info='')
    result = OutputResult(No=error['no'], Level=error['level'], Scope=error['scope'], Name=error['name'], Data=data.json, Comment=error['comment'])
    return result



"""Console script for ati18n."""
from pathlib import Path
import sys
import typer
from enum import Enum
from ati18n import Ati18n


app = typer.Typer(help="Awesome CLI user manager.")


class APPType(str, Enum):
    Java = "Java"
    Vue = "Vue"
    Flask = "Flask"


@app.command()
def check(path: Path = typer.Argument(..., help="应用路径"), 
    app_type: APPType = typer.Option("Java", "--type", "-t", help="specify the app type. 指定应用类型")):
    """Console script for ati18n.

    用来检查国际化是否有问题或可疑点的工具，支持Java应用、Vue应用、Flask应用中的多语言功能，检查其中是否有些可能的问题点，方便研发人员快速解决问题。"""
    arg_path={path}.pop()
    arg_app_type={app_type}.pop()
    ati = Ati18n(arg_app_type.value, arg_path)
    ati.start()
    return 0


if __name__ == "__main__":
    app()



__author__ = 'painterg'
__email__ = '22396997@qq.com'

import pandas as pd
from .common import *
from .utils import *
from . import *
import json
import os

class BaseCheck:

    count_status = []
    lang_status = []
    check_1001 = True
    check_1002 = True
    check_2001 = True

    """ 抽取文件数据抽取成字典格式：把每份文件的内容抽取成字典表 """
    def extract_dict(self, f):
        pass
        
    
    """ 数据内容会被整理成dataFrame格式进行后续的检查工作 """
    def output_result(self, df):
        pass

    """ check errors about the type of 1001 and 1002"""
    def check_file(self, app_type, file_name, value):
        return None


    """ check errors about the type of 1003"""
    def check_item(self, app_type, file_name, value):
        return None

    """ 
        path：i18n文件路径，不允许有子目录
        regex：获取i18n文件的正则表达式
    """
    def check(self,app_type, path, regex):
        df = self.load_file(path, regex)
        data = []
        ''' 获取df中每一行的数据，name为待翻译的key，value为行数据 '''
        for row in df.iterrows():
            key = row[0]
            value = row[1]
            ''' item为每一行的列数据，item[0]为列头，item[1]为列值 '''
            for item in value.items():
                data1 = self.check_file(app_type, item[0], item[1])
                data2 = self.check_item(app_type, item[0], key, item[1])
                if data1 is not None: data.append(data1)
                if data2 is not None: data.append(data2)
        self.output_result(data)        


    def load_file(self, path, regex):
        data = {}
        for file in path.glob(regex):
            assert file.is_file(), 'the directory contain sub directory'
            data[file.name] = self.convert_file(file)
        return pd.DataFrame(data)


    # 读取属性文件
    def convert_file(self, file):
        with file.open('r') as f:
            properties = self.extract_dict(f)
            return properties

    # 把df转化成json文件，并保留中文显示
    def to_json(self, df):
        return df.to_json(force_ascii=False)


class CheckJava(BaseCheck):

    def extract_dict(self, f):
        properties = {}
        for line in f:
            if line.find('=') > 0:
                strs = line.replace('\n', '').split('=')
                properties[strs[0]] = strs[1]
                    
        return properties

    
    def check_file(self, app_type, file_name, value):
        result = None
        if self.check_file and len(value) == 0  and self.count_status.count(file_name) == 0:
            self.count_status.append(file_name)
            result = out_template_file(file_name, ERROR_1001)
        elif self.check_1002 and len(value) > 0 and not determine_lang(file_name, value, app_type)[0] and self.lang_status.count(file_name) == 0:
            self.lang_status.append(file_name)
            result = out_template_file(file_name, ERROR_1002)
        return result.json if result is not None else None


    def check_item(self, app_type, file_name, key, value):
        result = None
        if self.check_2001 and len(value) > 0:
            lang_type = determine_lang(file_name, value, app_type)
            if not lang_type[0]:
                result = out_template_item(file_name, ERROR_2001, key, value, lang_type[1])
        return result.json if result is not None else None

    def output_result(self, data):
        app_path = os.getcwd()
        json_result = json.dumps(data, ensure_ascii=False)
        output_head = OutputHead(tools=OUTPUT_HEAD['tools'], version=OUTPUT_HEAD['version'], app_type=OUTPUT_HEAD['app_type'], app_path=app_path, datetime=get_now(), result=data)
        with open(app_path+ '/' + 'result.json', mode='w') as f:
            json.dump(output_head.json, f, ensure_ascii=False, indent=3)
            print('check is done...')


class CheckVue(BaseCheck):
    pass


class CheckFlask(BaseCheck):
    pass


"""Main module."""
from .aticheck import CheckJava
from .aticheck import CheckVue
from .aticheck import CheckFlask


class Ati18n:

    def __init__(self, app_type, path):
        self.app_type=app_type
        self.path=path
    

    def start(self):
        assert self.path.is_dir(), 'argument path is not directory: %s' % self.path
        if (self.app_type == 'Java'):
            check_obj = CheckJava()
            regex = '*.properties'
        elif (self.app_type == 'Vue'):
            check_obj = CheckVue()
            regex = ''
        elif (self.app_type == 'Flask'):
            check_obj = CheckFlask
            regex = ''
        check_obj.check(self.app_type, self.path, regex)




from enum import Enum


class DataType(str, Enum):
    File = "flie"
    Item = "item"


""" 定义一个pojo风格输出对象的基类 """
class OutputBase:
    
    __slots__ = ()

    def __init__(self, **kwargs):
        for k in kwargs.fromkeys(self.__slots__):
            if k in kwargs:
                setattr(self, k, kwargs[k])
    
    @property
    def json(self):
        return {s: getattr(self, s) for s in self.__slots__ if hasattr(self, s)}


class OutputHead(OutputBase):

    __slots__ = ('tools', 'version', 'app_path', 'app_type', 'datetime', 'result')

    def __init__(self, **kwargs):
        super().__init__(**kwargs)


class OutputResult(OutputBase):
   
    __slots__ = ('No', 'Level', 'Scope', 'Name', 'Data', 'Comment')

    def __init__(self, **kwargs):
        super().__init__(**kwargs)


class OutputDataSimple(OutputBase):

    __slots__ = ('type', 'file_path')

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

class OutputDataItem(OutputBase):

    __slots__ = ('type', 'key_name', 'key_value', 'lang_code', 'info', 'file_path')

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

